// sdl2X example application 2
//
// That's better than old one!

#include <time.h>
#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL_ttf.h>

#include "../sdl2X/hw_gp2x.h"
#include "../sdl2X/gp2x.h"
#include "../sdl2X/sdl_gp2x.h"
#include "../sdl2X/sdl_primitives_gp2x.h"
#include "../sdl2X/sdl_joystick_gp2x.h"
#include "../sdl2X/sdl_image_gp2x.h"
#include "../sdl2X/sdl_mixer_gp2x.h"
#include "../sdl2X/sdl_ttf_gp2x.h"

#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 240
#define SCREEN_DEPTH 16

#define BOX1_WIDTH 20
#define BOX1_HEIGHT 20

#define BOX2_WIDTH 50
#define BOX2_HEIGHT 50

// SDL Variables
SDL_Surface *screen, *text, *image, *gp2x;
TTF_Font *font, *font2;
Mix_Music *music;

// App variables
time_t t;
Uint8 r = 225, g = 225, b = 255;
int done = 0, volume = 1, fps = 0, frames = 0, freq;
char fpsmeter[16], info[128];
char *charge = "full", *battery = "off";

// Moving text
int text_x = 160, text_y = 9;

// Box variables
int box1_x = 160, box1_y = 120, box1_xdir = 0, box1_ydir = 0, box1_speed = 2;
int box2_x = 160, box2_y = 120, box2_xdir = 1, box2_ydir = 1, box2_speed = 1;

void MovingText(const char *c)
{
	// Render text
	text = RenderTextShaded(font, screen->format, c, 0xFFFFFF, 0x000000);

	// Move text
	text_x--;

	// If text has disappeared from screen
	if( text_x <= -text->w) text_x = 320;

	// Draw text
	DrawSurface(screen, text, text_x, text_y);

	// Free surface
	SDL_FreeSurface(text);
}

void MovingBox1()
{
	int i;

	for(i = 1; i <= box1_speed; i++)
	{
		// Move box
		if(box1_xdir == 0) box1_x--; else
		if(box1_xdir == 1) box1_x++;
		if(box1_ydir == 0) box1_y--; else
		if(box1_ydir == 1) box1_y++;

		// If box has collided, change direction
		if(box1_x == 0) box1_xdir = 1; else
		if(box1_x == 320 - BOX1_WIDTH) box1_xdir = 0;
		if(box1_y == 35) box1_ydir = 1; else
		if(box1_y == 205 - BOX1_HEIGHT) box1_ydir = 0;
	}

	// Draw box
	DrawBoxWithBorder(screen, box1_x, box1_y, BOX1_WIDTH, BOX1_HEIGHT, 0x0000FF, 1, 0x000000);
}

void MovingBox2()
{
	int i;

	for(i = 1; i <= box2_speed; i++)
	{
		// Move box
		if(box2_xdir == 0) box2_x--; else
		if(box2_xdir == 1) box2_x++;
		if(box2_ydir == 0) box2_y--; else
		if(box2_ydir == 1) box2_y++;

		// If box has collided, change direction
		if(box2_x == 0) box2_xdir = 1; else
		if(box2_x == 320 - BOX2_WIDTH) box2_xdir = 0;
		if(box2_y == 35) box2_ydir = 1; else
		if(box2_y == 205 - BOX2_HEIGHT) box2_ydir = 0;
	}

	// Draw box
	DrawBoxWithBorder(screen, box2_x, box2_y, BOX2_WIDTH, BOX2_HEIGHT, 0x509000, 1, 0x000000);
}

void LoadResources()
{
	// Load fonts
	font = LoadFont("font.ttf", 18);
	font2 = LoadFont("font.ttf", 12);

	// Load image
	image = LoadImage("image.bmp", 0xFF00FF);

	// GP2X 2006 Competition logo
	gp2x = LoadImage("gp2x.bmp", -1);

	// Load music
	music = LoadMusic("music.ogg");
	SetMusicVolume(volume);
	PlayMusic(music, 0);
}

void Controls()
{
	// Read joystick
	ReadJoystick(1);

	// Change background color
	if(joy2 | joy3 | joy4)
	{
		if(r < 255) r++;
		if(g < 255) g++;
		if(b < 255) b++;
	}
	if(joy6 | joy7 | joy8)
	{
		if(r > 0) r--;
		if(g > 0) g--;
		if(b > 0) b--;
	}

	// On/Off screen and battery LED
	if(joyl)
	{
		ShutdownScreen();
		while(joyl)
			ReadJoystick(1);
	}
	if(joyr)
	{
		BatteryLED();
		while(joyr)
			ReadJoystick(1);
	}

	// Change 920t clock frequency
	if(joyb)
	{
		freq += 2;
		Set920Clock(freq);
		SDL_Delay(100);
	}
	if(joya)
	{
		freq -= 2;
		Set920Clock(freq);
		SDL_Delay(100);
	}

	// Change music volume
	if(joyvoldown)
		if(volume > 0)
		{
			volume--;
			SetMusicVolume(volume);
		}
	if(joyvolup)
	{
		volume++;
		SetMusicVolume(volume);
	}

	// Capture screenshot
	if(joyy) SaveScreenshot(screen);

	if(joystart) done = 1;
}

void Draw()
{
	// Draw background
	DrawBox(screen, 0, 0, 320, 240, SDL_MapRGB(screen->format, r, g, b));

	// Draw edges
	DrawBox(screen, 0, 0, 320, 35, 0x000000);
	DrawBox(screen, 0, 205, 320, 35, 0x000000);

	// Draw image
	DrawSurfaceCenter(screen, image, 160, 120);

	// Moving boxes
	MovingBox1();
	MovingBox2();

	// Moving text
	MovingText("sdl2X App v2 :: GP2X :: Waninkoko");

	// Draw FPS
	sprintf(fpsmeter, "%i FPS", fps);
	DrawTextShaded(screen, font2, fpsmeter, 160, 215, 0xFFFFFF, 0x000000);

	// Draw GP2X info
	sprintf(info, "CPU: %i Mhz :: Battery LED: %s :: Charge: %s", freq, battery, charge);
	DrawTextShaded(screen, font2, info, 160, 230, 0xFFFFFF, 0x000000);

	// Draw GP2X 2006 Competition logo
	DrawSurface(screen, gp2x, (320 - gp2x->w), 0);
}

void MainMenu()
{
	// Load resources
	LoadResources();

	while(!done)
	{
		// Get time to calculate FPS
		if(frames == 0) t = time(NULL);

		// Controls
		Controls();

		// Get battery LED state and battery charge
		if(BatteryLEDState()) battery = "on"; else battery = "off";
		switch(GetBatteryCharge())
		{
			case 0:
				charge = "empty";
				break;

			case 1:
				charge = "medium";
				break;

			case 2:
				charge = "full";
				break;

			case 3:
				charge = "adapter";
				break;
		}

		// Add one more frame and if has passed a second, calculate frames per second
		frames++;
		if(time(NULL) - t >= 1)
		{
			fps = frames;
			frames = 0;
		}

		// Draw screen
		Draw();

		SDL_Flip(screen);
	}

	StopMusic(music);

	// Close fonts
	CloseFont(font);
	CloseFont(font2);
}

int main(int argc, char *argv[])
{
	// Initialize SDL
	screen = InitSDL(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_DEPTH);

	// Open joystick device
	JoystickOpen(0);

	// Prepare joystick
	PrepareJoystick();

	// Initialize SDL_mixer
	InitSDL_mixer(22050, 2, 512);

	// Initialize SDL_ttf
	InitSDL_TTF();

	// Get 920t clock frequency
	freq = Get920Clock();

	// Main Menu
	MainMenu();

	// Quit SDL
	QuitSDL();

	// Return to GP2X menu
	ReturnGP2X();

	return 0;
}
